package com.elpuig.androidam.activities;

import com.elpuig.androidam.core.Info;
import com.elpuig.androidam.parsers.ParserLogin;
import com.elpuig.attendancemanager.R;

import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

/**
 * Clase Config (Activity)
 * 
 * Clase que gestiona la configuración de la aplicación.
 * Este activity recoge los datos de usuario y contraseña 
 * del profesor, y la url donde se encuentra el servidor de 
 * Attendance Manager. Los datos se almacenan en las
 * Shared Preferences.
 * 
 * 
 * @author Mario Álvarez <malvarez@iespuigcastellar.xeill.net>
 * @author Adrián García <agarcia2@iespuigcastellar.xeill.net>
 * @author Edgar Moreno <emoreno@iespuigcastellar.xeill.net>
 *
 */
public class Config extends Activity implements OnClickListener{
	public static final String PREFS = "MyInfo";
	 EditText etName = null;
     EditText etPass = null;
     EditText etUrl = null;
     Button bConfirm = null;
     Info inf = new Info();
	public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.config);
        etName = (EditText) findViewById(R.id.etName);
        etPass = (EditText) findViewById(R.id.etPass);
        etUrl = (EditText) findViewById(R.id.etUrl);
        bConfirm = (Button) findViewById(R.id.bConfirm);
        bConfirm.setOnClickListener(this);
	
	}

	@Override
	public void onClick(View v) {
		 inf.setUser(etName.getText().toString());
		 inf.setPassword(etPass.getText().toString());
		 inf.setUrl(etUrl.getText().toString());
	    
	     
	     try{
	    	 ParserLogin parser = new ParserLogin(inf.getUrl() + "/AttendanceManager/AndroidServlet?operation=0&name=" + inf.getUser() + "&password=" + inf.getPassword());
	    	 Info result = new Info(); 
	    	 result = parser.parse();
	    	 if (result.getUser() != "null"){
	    		   // We need an Editor object to make preference changes.
	    	      // All objects are from android.context.Context
	    	      SharedPreferences settings =  getSharedPreferences(PREFS, 0);
	    	      SharedPreferences.Editor editor = settings.edit();
	    	      editor.putString("name", inf.getUser()).commit();
	    	      editor.putString("password", inf.getPassword()).commit();
	    	      editor.putString("url", inf.getUrl()).commit();
	    	      editor.putBoolean("tutor", result.isTutor()).commit();
		    	 startTeachersMain();
		     }
		     else{
		    	 Toast.makeText( getApplicationContext(), "Error en el login", Toast.LENGTH_SHORT).show();
		     }
	    	 
			}catch (Exception e){
				Toast.makeText( getApplicationContext(), "Error al conectar al servidor", Toast.LENGTH_SHORT).show();
				}
	    
	     
		
	}
	public void startTeachersMain(){
		try{
			Intent intent = new Intent(this, TeachersMain.class);
			startActivity(intent);
			this.finish();
		}catch (Exception e){
			Toast.makeText( getApplicationContext(), "Error al abrir la pantalla principal", Toast.LENGTH_SHORT).show();
			}
		
	}

}
