package com.elpuig.androidam.activities;


import java.util.ArrayList;
import java.util.Calendar;

import com.elpuig.androidam.adapters.StudentAdapter;
import com.elpuig.androidam.core.Classblock;
import com.elpuig.androidam.core.Info;
import com.elpuig.androidam.core.Misses;
import com.elpuig.androidam.core.Student;
import com.elpuig.androidam.parsers.ParserClassblocks;
import com.elpuig.androidam.parsers.ParserConfirmation;
import com.elpuig.androidam.parsers.ParserMisses;
import com.elpuig.androidam.parsers.ParserStudent;
import com.elpuig.attendancemanager.R;


import android.app.Activity;
import android.app.AlertDialog;
import android.app.DatePickerDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.os.Handler;
import android.provider.Settings;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.ArrayAdapter;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

/**
 * Clase TeachersMain (Activity)
 * 
 * Clase principal de la aplicación, que gestiona las faltas
 * de asistencia en los diferentes classblocks del profesor, 
 * según el día indicado.
 * 
 * Ademas permite acceder a la tutoria personal del profesor y 
 * editar los datos de configuración a traves del menú, asi como
 * llamar y enviar SMS al alumno seleccionado.
 * 
 * 
 * @author Mario Álvarez <malvarez@iespuigcastellar.xeill.net>
 * @author Adrián García <agarcia2@iespuigcastellar.xeill.net>
 * @author Edgar Moreno <emoreno@iespuigcastellar.xeill.net>
 *
 */
public class TeachersMain extends Activity{
	public static final String PREFS = "MyInfo";
	public static Info inf = new Info();

 	private static final int DELETE_DATA = Menu.FIRST;
    private static final int EXIT = Menu.FIRST + 1;
    private static final int TUTORIA = Menu.FIRST + 2;
    private static final int DATE_DIALOG_ID = 0;
    
    private ArrayList<Student> listaClass = null;
    private Context appcontext = this;
    private ListView lv = null;
    private Button mPickDate;
    private String selectedDate = null;
    public static Classblock selectedClassblock = null; 
    private Student selectedStudent = null;
	
	private int mYear;
    private int mMonth;
    private int mDay;

   	
    
    
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.main);
        lv=(ListView) findViewById(R.id.listview);
        
        restorePreferences();

        if (inf.getUser().equals("")){
        	initConfig();
        }
        else{
        	 try{
        		 if (isConnectionAvailable()==true){
                 	
        			initDateButton();
                  	initSpinner();
                  	initListView();  
    
        		 }
        		 else 
        		 { 
        			 Toast.makeText( getApplicationContext(), "No dispone de conexion a internet", Toast.LENGTH_SHORT).show();
        		 }
        		 
             }catch(Exception e){
             	Toast.makeText( getApplicationContext(), "Error de conexión, comprueba los datos de configuración", Toast.LENGTH_SHORT).show();
             }
        } 
    	
    }
    
    @Override
    protected Dialog onCreateDialog(int id) {
        switch (id) {
        case DATE_DIALOG_ID:
            return new DatePickerDialog(this,mDateSetListener,mYear, mMonth, mDay);
                }
       
        return null;
    }
    
    
    private DatePickerDialog.OnDateSetListener mDateSetListener =
        new DatePickerDialog.OnDateSetListener() {

            public void onDateSet(DatePicker view, int year, 
                                  int monthOfYear, int dayOfMonth) {
                mYear = year;
                mMonth = monthOfYear;
                mDay = dayOfMonth;
                selectedDate = mDay+"-"+(mMonth+1)+"-"+mYear; 
                lv.setAdapter(null);
                initSpinner();
                updateDisplay();
            }
        };
        
        
   /**
 * Este método lanza un nuevo activity Tutor.
 * 
 * 
 */
private void initTutor(){
	   try{
			Intent intent = new Intent(this, Tutor.class);
			startActivity(intent);
		}catch (Exception e){
			Toast.makeText( getApplicationContext(), "Error al abrir Tutoria", Toast.LENGTH_SHORT).show();
			}
	   
	   
   }
        

   
    /**
     * Método que inicializa el spinner de classblocks, con los datos que devuelve
     * el servidor.
     */
    private void initSpinner(){
    	
    	
    	Spinner spClassblocks = (Spinner) findViewById( R.id.spClassblocks);
    	spClassblocks.setAdapter(null);
    	lv.setAdapter(null);
     	ParserClassblocks parser = new ParserClassblocks (inf.getUrl() + "/AttendanceManager/AndroidServlet?operation=1&name=" + inf.getUser() + "&password=" + inf.getPassword()+ "&date=" + selectedDate);
    	ArrayList<Classblock> listaClassblocks = parser.parse();
    	
    	
        ArrayAdapter<Classblock> adapter = new ArrayAdapter<Classblock>(
                this, android.R.layout.simple_spinner_item, listaClassblocks);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

        
        spClassblocks.setAdapter(adapter);
    
        spClassblocks.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) {

            	//Barra de progreso mientras se cargan los datos.
            	final ProgressDialog dialogp;
            	dialogp = new ProgressDialog(appcontext);
            	dialogp.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
            	dialogp.setMessage("Cargando datos...");
            	dialogp.setCancelable(false);
            	dialogp.show(); //Se muestra.
            	Classblock aux = (Classblock) parent.getItemAtPosition(pos);
	       	    selectedClassblock = aux;
	       	    dialogp.incrementProgressBy(25); // incremento de 25
	       	 
	       	    //Nuevo thread para cargar los datos del servidor.
	         	 new Handler().postDelayed(new Runnable() {
         	        @Override
         	        public void run() {
         	        	dialogp.incrementProgressBy(25); // incremento de 25
         	        	ParserStudent parserclass = new ParserStudent (inf.getUrl() + "/AttendanceManager/AndroidServlet?operation=2&name=" + inf.getUser() + "&password=" + inf.getPassword() + "&idgroup=" +selectedClassblock.getIdgroup() + "&idclassblock=" +selectedClassblock.getId()+ "&date=" + selectedDate  );
         	        	listaClass = parserclass.parse();
         	        	dialogp.incrementProgressBy(25); // incremento de 25
         	         	lv.setAdapter(new StudentAdapter(TeachersMain.this, R.layout.list_item, listaClass));
         	         	dialogp.incrementProgressBy(25); // incremento de 25
         	         	dialogp.dismiss(); //Se cierra el progress dialog.

         	        }
         	    }, 100);
	       	     
 	         	
 	            }
 	            public void onNothingSelected(AdapterView<?> parent) {
 	            	//Code
 	            }
 	        });
            		
         
    	 
     }
     
    
    /**
     * Método que inicializa el listview con sus listeners.
     * 
     */
    private void initListView(){
    	
	
    	
        lv.setOnItemLongClickListener(new OnItemLongClickListener(){
        
			@Override
			public boolean onItemLongClick(AdapterView<?> parent, View arg1,
					int position, long arg3) {
				
				final Student aux = (Student) parent.getItemAtPosition(position);
				selectedStudent = aux;
				
				//Nuevo dialog de información del alumno
				final Dialog dialog_info = new Dialog(appcontext);
	            dialog_info.setContentView(R.layout.profile);
	            dialog_info.setTitle("" + aux.toString());
	            dialog_info.setCancelable(true);
	            
	            //Se carga la información de faltas de asistencia del alumno seleccionado.
	            ParserMisses parserM = new ParserMisses((inf.getUrl() + "/AttendanceManager/AndroidServlet?operation=3&name=" + inf.getUser() + "&password=" + inf.getPassword() + "&idsubject=" +selectedClassblock.getIdsubject()+ "&idstudent="+aux.getId()));
	            Misses misses = parserM.parse();
	            TextView phonenumber = (TextView) dialog_info.findViewById(R.id.textView_phone);
	            phonenumber.setText("Teléfono: " +aux.getSmsPhone());
	            
	            TextView expulsiont = (TextView) dialog_info.findViewById(R.id.textView_expulsion);
	            expulsiont.setText("Expulsiones: " +misses.getClassblocksexpulsion());
	            
	            TextView missest = (TextView) dialog_info.findViewById(R.id.textView_misses);
	            missest.setText("Faltas: " +misses.getClassblocksmisses());
	            
	            TextView excusedt = (TextView) dialog_info.findViewById(R.id.textView_excused);
	            excusedt.setText("Justificadas: " +misses.getClassblocksexcused());
	            
	            TextView delayt = (TextView) dialog_info.findViewById(R.id.textView_delay);
	            delayt.setText("Retrasos: " +misses.getClassblocksdelay());
	            
	            Button bCall = (Button) dialog_info.findViewById(R.id.bCall);
	            Button bSMS = (Button) dialog_info.findViewById(R.id.bSMS);
	            Drawable ic = getResources().getDrawable(R.drawable.stub);
	            ImageView image = (ImageView) dialog_info.findViewById(R.id.imageView_profile);
	            byte[] b= selectedStudent.getPhoto();
	           
	            if (b==null){
	            	//Se muestra la imagen predeterminada
	            	image.setImageDrawable(ic);
	            }
	            else{
	            	//Se muestra la imagen descargada del servidor.
	        	   Drawable imagenfinal = null;
	        	   imagenfinal =  new BitmapDrawable(BitmapFactory.decodeByteArray(b, 0, b.length)); 
	               
	               image.setImageDrawable(imagenfinal);
	           }
	            dialog_info.show();
	            
	            
	            //Llama al alumno seleccionado
	            bCall.setOnClickListener(new View.OnClickListener() {
	                public void onClick(View v) {
	                    callToNumber(aux.getSmsPhone());
	                }
	            });
	            bSMS.setOnClickListener(new View.OnClickListener() {
	                public void onClick(View v) {
	                	dialog_info.dismiss(); //Cerramos el anterior dialog de información.
	                	
	                	//Nuevo dialog para el envio de SMS
	                	final Dialog dialog_sms = new Dialog(appcontext);
	                	 dialog_sms.setContentView(R.layout.sms_view);
	                	 dialog_sms.setCancelable(true);
	                	 dialog_sms.setTitle("Mensaje para: " + aux.toString());
	                	 Button benviasms = (Button) dialog_sms.findViewById(R.id.benviarSms);
	                	 final TextView tmessage = (TextView) dialog_sms.findViewById(R.id.textosms);   	 
	                
	                     dialog_sms.show();
	                     

	                     benviasms.setOnClickListener(new View.OnClickListener() {
		     	                public void onClick(View v) {
		     	                	
		     	                	//Se pasan los datos al metodo encargado de mandar el SMS
		     	                	CharSequence cs = tmessage.getText().toString();
		     	                	
				                     boolean b = enviaSms(cs,selectedStudent.getSmsPhone());
				                     dialog_sms.dismiss();
				                     if (b){
				                  		
				                  		Toast.makeText(appcontext, "SMS ENVIADO", Toast.LENGTH_SHORT).show();
				                  		
				                  	}
				                  	else{
				                  		Toast.makeText(appcontext, "ERROR EN EL ENVIO", Toast.LENGTH_SHORT).show();
				                  	}  	
		     	                }

								
		     	            });
	                     
	    	            
	                }
	            });
				
				
				
				return false;
			}
        	
        });
        
        
        lv.setOnItemClickListener(new OnItemClickListener() {
            public void onItemClick(AdapterView<?> parent, View view,
                int position, long id) {
            	Student aux = (Student) parent.getItemAtPosition(position);
            	addMiss(aux);
            	
            }
          });
    	
    }

    /**
     * Método que envia un SMS a traves del servidor de AM.
     * Se reciben como parametros el numero de teléfono y el texto a enviar.
     * 
     * 
     * 
     * @param msg
     * @param smsPhone
     * @return bool
     */
    private boolean enviaSms(CharSequence msg, String smsPhone) {
    	Boolean bool =false;
    	try{
    	ParserConfirmation parsermsg = new ParserConfirmation (inf.getUrl() + "/AttendanceManager/AndroidServlet?operation=4&name=" + inf.getUser() + "&password=" + inf.getPassword()  + "&phonenumber=" +smsPhone+ "&msg=" + msg);
     	bool = parsermsg.parse();
     	
    	}catch(Exception e) { }
    	return bool;
	}
    
    
    /**PARA POSIBLES MEJORAS:
     * 
     * -Método que establece el proxy del sistema. No funciona correctamente.
     * -No se utiliza.
     * 
     */
    @SuppressWarnings("unused")
	private void enableProxy(){
    	try{
    
    	Settings.System.putString(this.getContentResolver(), Settings.System.HTTP_PROXY, "http://proxy:8080");    
    	
    	}catch (Exception ex){
    		Log.e("PROPERTY", Settings.System.getString(getContentResolver(), Settings.System.HTTP_PROXY) + "3");
    	}
    	
    	
    }
    
    /**PARA POSIBLES MEJORAS:
     * 
     * -Método que elimina datos del proxy del sistema. No funciona correctamente.
     * -No se utiliza.
     * 
     */
    @SuppressWarnings("unused")
	private void disableProxy(){
    	try{
    
    	Settings.System.putString(this.getContentResolver(), Settings.System.HTTP_PROXY, "");
    	}catch (Exception ex){
    	}
    	
    	
    }
       
    
    /**
     * Método que añade la falta correspondiente al alumno que se pasa por parametro.
     * El estado del alumno se incrementa en 1 y se envia el tipo de falta al servidor.
     * Si su estado es 4, el estado vuelve a 0 y se eliminan las faltas.
     * 
     * @param student
     */
    private void addMiss(Student student){
    	
    	Integer state = student.getState();
    	Integer newstate;
    	if (state<4){
    		newstate = state + 1;
    	}
    	else{
    		newstate = 0;
    	}
    	ParserConfirmation parsermiss = new ParserConfirmation (inf.getUrl() + "/AttendanceManager/AndroidServlet?operation=5&name=" + inf.getUser() + "&password=" + inf.getPassword()  + "&idclassblock=" +selectedClassblock.getId()+ "&date=" + selectedDate + "&idstudent=" + student.getId() + "&level=" + newstate );
     	Boolean bool = parsermiss.parse();
     	if (bool == true){
     		student.setState(newstate);
        	((BaseAdapter) lv.getAdapter()).notifyDataSetChanged(); 
     	}
     	else{ 
     		Toast.makeText(appcontext, "No se ha añadido la falta", Toast.LENGTH_SHORT).show();
     	}
    	
    }
    
    
    /**
     * Método que inicializa el boton de fecha.
     * 
     */
    private void initDateButton(){

           
           mPickDate = (Button) findViewById(R.id.pickDate);

           // add a click listener to the button
           mPickDate.setOnClickListener(new View.OnClickListener() {
               public void onClick(View v) {
                   showDialog(DATE_DIALOG_ID);
               }
           });

           // get the current date
           final Calendar c = Calendar.getInstance();
           mYear = c.get(Calendar.YEAR);
           mMonth = c.get(Calendar.MONTH);
           mDay = c.get(Calendar.DAY_OF_MONTH);
           selectedDate = mDay+"-"+(mMonth+1)+"-"+mYear;

           // display the current date
           updateDisplay();
           
    	
    }
    
    //Recarga la visualizacion de la fecha en el boton.
    private void updateDisplay() {
    	mPickDate.setText(selectedDate);
    }
    
    
    /**
     * Lanza un nuevo activity de configuración.
     * 
     */
    private void initConfig(){
    	
    	try{
			Intent intent = new Intent(this, Config.class);
			startActivity(intent);
			this.finish();
		}catch (Exception e){
			Toast.makeText( getApplicationContext(), "Error al abrir la configuración", Toast.LENGTH_SHORT).show();
			}
    }
    
    /**
     * Método que se encarga de cargar en un objeto de tipo INFO la información 
     * almacenada en las Shared Preferences.
     * 
     */
    private void restorePreferences(){

        SharedPreferences settings = getSharedPreferences(PREFS, 0);
        String aux = settings.getString("name", "");
        inf.setUser(aux);
        aux = settings.getString("password", "");
        inf.setPassword(aux);
        aux=settings.getString("url", "");
        inf.setUrl(aux);
        Boolean istutor=settings.getBoolean("tutor", false);
        inf.setTutor(istutor);
    	
    }
    
    /**
     * Método que se encarga de borrar la información 
     * almacenada en las Shared Preferences. Despues, lanza
     * un nuevo activity de configuración.
     * 
     */
    private void deletePreferences(){
    	 
	      AlertDialog.Builder builder = new AlertDialog.Builder(this);
	      builder.setMessage("Seguro que desea borrar los datos actuales?")
	             .setCancelable(false)
	             .setPositiveButton("Si", new DialogInterface.OnClickListener() {
	                 public void onClick(DialogInterface dialog, int id) {
	                	 SharedPreferences settings =  getSharedPreferences(PREFS, 0);
	           	      SharedPreferences.Editor editor = settings.edit();
	           	      editor.putString("name","").commit();
	           	      editor.putString("password","").commit();
	           	      editor.putString("url","").commit();
	           	      editor.putBoolean("tutor", false).commit();
	           	      initConfig();
	                 }
	             })
	             .setNegativeButton("No", new DialogInterface.OnClickListener() {
	                 public void onClick(DialogInterface dialog, int id) {
	                      dialog.cancel();
	                 }
	             });
	      AlertDialog alert = builder.create();
	      alert.show();
    }	
    
    
    
    /**
     * Llama al numero indicado lanzando un nuevo intent.
     * 
     * @param number
     */
    public void callToNumber(String number) {
    	 Intent intent = new Intent(Intent.ACTION_CALL, Uri.parse("tel:" + number));
    	 startActivity(intent);
    }
    
    
    /**
     * Metodo que habilita el wifi del telefono.
     * No se utiliza por el momento, PARA POSIBLES MEJORAS.
     */
    public void enableWifi(){
    	WifiManager wm = (WifiManager) getSystemService(Context.WIFI_SERVICE);
     	if (wm.isWifiEnabled()== false){
     		wm.setWifiEnabled(true);
     		Toast.makeText( getApplicationContext(), "Wifi enabled", Toast.LENGTH_SHORT).show();
     	}    
    	
    }
 
    
    /**
     * Para saber si hay conexión disponible. Devuelve true o false.
     * 
     * @return result
     */
    public boolean isConnectionAvailable(){
    	Boolean result = false;
    	
    	ConnectivityManager conn_manager = (ConnectivityManager) this.getSystemService(Context.CONNECTIVITY_SERVICE);
    	NetworkInfo network_info = conn_manager.getActiveNetworkInfo();
    	if ( network_info != null && network_info.isConnected() ) {
    	    if ( network_info.getType() == ConnectivityManager.TYPE_WIFI )
    	        result = true;
    	    else
    	       result = true;
    	}
    	return result;
    	
    }
    
    @Override
    public  boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        menu.add(Menu.NONE, DELETE_DATA, 0,"Editar configuración");
        menu.add(Menu.NONE, TUTORIA, 1,"Tutoria");
        menu.add(Menu.NONE, EXIT, 2, "Salir");
        return true;
    } 
    
    @Override
    public boolean onMenuItemSelected(int featureId, MenuItem item) {
     super.onMenuItemSelected(featureId, item);
           switch(item.getItemId()) {
           case TUTORIA:
        	   initTutor();
               break;
           case DELETE_DATA:
        	   deletePreferences();
               break;
           case EXIT:
        	   finish();
               break;
           }
           return true;
    }

	
    
    
}