package com.elpuig.androidam.activities;

import java.util.ArrayList;

import com.elpuig.androidam.adapters.StudentAdapter;
import com.elpuig.androidam.core.Group;
import com.elpuig.androidam.core.Info;
import com.elpuig.androidam.core.Misses;
import com.elpuig.androidam.core.Student;
import com.elpuig.androidam.parsers.ParserConfirmation;
import com.elpuig.androidam.parsers.ParserGroups;
import com.elpuig.androidam.parsers.ParserMisses;
import com.elpuig.androidam.parsers.ParserStudent;
import com.elpuig.attendancemanager.R;

import android.app.Activity;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemClickListener;

/**
 * Clase que gestiona la tutoria de el profesor. Layout muy similar
 * al de TeachersMain, con spinner de seleccion de grupo (en el caso
 * en el que el profesor sea tutor de mas de un grupo), y lista de 
 * alumnos del grupo, para poder revisar sus faltas totales, asi como
 * llamarlos o mandarles un SMS. 
 * 
 * Si el profesor no es tutor de ningún grupo, no podra acceder.
 * 
 * 
 * @author Mario Álvarez <malvarez@iespuigcastellar.xeill.net>
 * @author Adrián García <agarcia2@iespuigcastellar.xeill.net>
 * @author Edgar Moreno <emoreno@iespuigcastellar.xeill.net>
 *
 */
public class Tutor extends Activity{
	public static final String PREFS = "MyInfo";
	public static Info inf = new Info();
	private Context appcontext = this;

	private Student selectedStudent = null;
	
	@Override
	public void onCreate(Bundle savedInstanceState){
		super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.tutor);
        restorePreferences();
        if (inf.getUser().equals("")){
        	Toast.makeText(this, "Edite la configuración", Toast.LENGTH_SHORT).show();
        }
        if (inf.isTutor()){
        	initSpinner();
        }
        
        else{
        	Toast.makeText(this, "Actualmente no es tutor de ningún grupo", Toast.LENGTH_SHORT).show();
        	this.finish();
        }

        
	}
	
	public void initSpinner(){
		ParserGroups parser = new ParserGroups (inf.getUrl() + "/AttendanceManager/AndroidServlet?operation=6&name=" + inf.getUser() + "&password=" + inf.getPassword());
    	ArrayList<Group> listaGroup = parser.parse();

    		Spinner spGroups = (Spinner) findViewById( R.id.spTutor);
          ArrayAdapter<Group> adapter = new ArrayAdapter<Group>(
                  this, android.R.layout.simple_spinner_item, listaGroup);
          adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
          spGroups.setAdapter(adapter);
          
         
          
          
          
          
          spGroups.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
              public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) {
              
            	//Barra de progreso mientras se cargan los datos.
              	final ProgressDialog dialogp;
              	dialogp = new ProgressDialog(appcontext);
              	dialogp.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
              	dialogp.setMessage("Cargando datos...");
              	dialogp.setCancelable(false);
              	dialogp.show(); //Se muestra.
              	dialogp.incrementProgressBy(25); // incremento de 25

         	    final Group aux = (Group) parent.getItemAtPosition(pos);
         	    final ListView lv = (ListView)findViewById(R.id.listViewTutor);
         	 //Nuevo thread para cargar los datos del servidor.
	         	 new Handler().postDelayed(new Runnable() {
        	        @Override
        	        public void run() {
		         	    ArrayList<Student> listaClass = new ArrayList<Student>();
		         	    dialogp.incrementProgressBy(25); // incremento de 25
		         	    ParserStudent parserclass = new ParserStudent (inf.getUrl() + "/AttendanceManager/AndroidServlet?operation=7&name=" + inf.getUser() + "&password=" + inf.getPassword() + "&idgroup=" +aux.getId());
		         	    listaClass = parserclass.parse();
		         	    dialogp.incrementProgressBy(25); // incremento de 25
		         	   
		         	    lv.setAdapter(new StudentAdapter(Tutor.this, R.layout.list_item, listaClass));
		         	    dialogp.incrementProgressBy(25); // incremento de 25
		         	    dialogp.dismiss();
        	        }
	         	    }, 100);
         	    
         	    
         	    
         	   lv.setOnItemClickListener(new OnItemClickListener() {
                   public void onItemClick(AdapterView<?> parent, View view,
                       int position, long id) {
                	   final Student aux = (Student) parent.getItemAtPosition(position);
       				selectedStudent = aux;
       				
       				//Nuevo dialog de información del alumno
       				final Dialog dialog_info = new Dialog(appcontext);
       	            dialog_info.setContentView(R.layout.profile);
       	            dialog_info.setTitle("" + aux.toString());
       	            dialog_info.setCancelable(true);
       	            
       	            //Se carga la información de faltas de asistencia del alumno seleccionado.
       	            ParserMisses parserM = new ParserMisses((inf.getUrl() + "/AttendanceManager/AndroidServlet?operation=3&name=" + inf.getUser() + "&password=" + inf.getPassword() + "&idsubject=" +TeachersMain.selectedClassblock.getIdsubject()+ "&idstudent="+aux.getId()));
       	            Misses misses = parserM.parse();
       	            TextView phonenumber = (TextView) dialog_info.findViewById(R.id.textView_phone);
       	            phonenumber.setText("Teléfono: " +aux.getSmsPhone());
       	            
       	            TextView expulsiont = (TextView) dialog_info.findViewById(R.id.textView_expulsion);
       	            expulsiont.setText("Expulsiones: " +misses.getTotalexpulsion());
       	            
       	            TextView missest = (TextView) dialog_info.findViewById(R.id.textView_misses);
       	            missest.setText("Faltas: " +misses.getTotalmisses());
       	            
       	            TextView excusedt = (TextView) dialog_info.findViewById(R.id.textView_excused);
       	            excusedt.setText("Justificadas: " +misses.getTotalexcused());
       	            
       	            TextView delayt = (TextView) dialog_info.findViewById(R.id.textView_delay);
       	            delayt.setText("Retrasos: " +misses.getTotaldelay());
       	            
       	            Button bCall = (Button) dialog_info.findViewById(R.id.bCall);
       	            Button bSMS = (Button) dialog_info.findViewById(R.id.bSMS);
       	            Drawable ic = getResources().getDrawable(R.drawable.stub);
       	            ImageView image = (ImageView) dialog_info.findViewById(R.id.imageView_profile);
       	            byte[] b= selectedStudent.getPhoto();
       	           
       	            if (b==null){
       	            	//Se muestra la imagen predeterminada
       	            	image.setImageDrawable(ic);
       	            }
       	            else{
       	            	//Se muestra la imagen descargada del servidor.
       	        	   Drawable imagenfinal = null;
       	        	   imagenfinal =  new BitmapDrawable(BitmapFactory.decodeByteArray(b, 0, b.length)); 
       	               
       	               image.setImageDrawable(imagenfinal);
       	           }
       	            dialog_info.show();
       	            
       	            
       	            //Llama al alumno seleccionado
       	            bCall.setOnClickListener(new View.OnClickListener() {
       	                public void onClick(View v) {
       	                    callToNumber(aux.getSmsPhone());
       	                }
       	            });
       	            bSMS.setOnClickListener(new View.OnClickListener() {
       	                public void onClick(View v) {
       	                	dialog_info.dismiss(); //Cerramos el anterior dialog de información.
       	                	
       	                	//Nuevo dialog para el envio de SMS
       	                	final Dialog dialog_sms = new Dialog(appcontext);
       	                	 dialog_sms.setContentView(R.layout.sms_view);
       	                	 dialog_sms.setCancelable(true);
       	                	 dialog_sms.setTitle("Mensaje para: " + aux.toString());
       	                	 Button benviasms = (Button) dialog_sms.findViewById(R.id.benviarSms);
       	                	 final TextView tmessage = (TextView) dialog_sms.findViewById(R.id.textosms);   	 
       	                
       	                     dialog_sms.show();
       	                     

       	                     benviasms.setOnClickListener(new View.OnClickListener() {
       		     	                public void onClick(View v) {
       		     	                	
       		     	                	//Se pasan los datos al metodo encargado de mandar el SMS
       				                     boolean b = enviaSms(tmessage.getText().toString(),selectedStudent.getSmsPhone());
       				                     dialog_sms.dismiss();
       				                     if (b){
       				                  		
       				                  		Toast.makeText(appcontext, "SMS ENVIADO", Toast.LENGTH_SHORT).show();
       				                  		
       				                  	}
       				                  	else{
       				                  		Toast.makeText(appcontext, "ERROR EN EL ENVIO", Toast.LENGTH_SHORT).show();
       				                  	}  	
       		     	                }

       								
       		     	            });
       	                     
       	    	            
       	                }
       	            });
       				
       				
       				
       
       			}
               	
               });
                  
                   	
                   }

			@Override
			public void onNothingSelected(AdapterView<?> arg0) {
				// TODO Auto-generated method stub
				
			}
                 });
         	    
         	    
         	    
         	    
              


    	}
    		
	
	  /**
     * Llama al numero indicado lanzando un nuevo intent.
     * 
     * @param number
     */
    public void callToNumber(String number) {
    	 Intent intent = new Intent(Intent.ACTION_CALL, Uri.parse("tel:" + number));
    	 startActivity(intent);
    }
    
    
    
    /**
     * Método que envia un SMS a traves del servidor de AM.
     * Se reciben como parametros el numero de teléfono y el texto a enviar.
     * 
     * 
     * 
     * @param msg
     * @param smsPhone
     * @return bool
     */
    private boolean enviaSms(String msg, String smsPhone) {
    	ParserConfirmation parsermsg = new ParserConfirmation (inf.getUrl() + "/AttendanceManager/AndroidServlet?operation=4&name=" + inf.getUser() + "&password=" + inf.getPassword()  + "&phonenumber=" +smsPhone+ "&msg=" + msg);
     	Boolean bool = parsermsg.parse();
     	return bool;
		
	}
	
	
			
	private void restorePreferences(){
    	// Restore preferences
        SharedPreferences settings = getSharedPreferences(PREFS, 0);
        String aux = settings.getString("name", "");
        inf.setUser(aux);
        aux = settings.getString("password", "");
        inf.setPassword(aux);
        aux=settings.getString("url", "");
        inf.setUrl(aux);
        Boolean istutor=settings.getBoolean("tutor", false);
        inf.setTutor(istutor);
    	
    }
	

}
