package com.elpuig.androidam.parsers;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.elpuig.androidam.core.Student;

public class ParserStudent
{
    private URL Url;
 
    public ParserStudent(String url)
    {
        try
        {
            this.Url = new URL(url);
        }
        catch (MalformedURLException e)
        {
            throw new RuntimeException(e);
        }
    }
 
    public ArrayList<Student> parse()
    {
        //Instanciamos la fábrica para DOM
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        	ArrayList<Student> classes = new ArrayList<Student>();
 
        try
        {
            //Creamos un nuevo parser DOM
            DocumentBuilder builder = factory.newDocumentBuilder();
 
            //Realizamos lalectura completa del XML
            Document dom = builder.parse(this.getInputStream());
 
            //Nos posicionamos en el nodo principal del árbol
            Element root = dom.getDocumentElement();
 
            //Localizamos todos los elementos
            NodeList items = root.getElementsByTagName("student");
 
            //Recorremos la lista
            for (int i=0; i<items.getLength(); i++)
            {
                Student aux = new Student();
 
                //Obtenemos el item actual
                Node item = items.item(i);
 
                //Obtenemos la lista de datos
                NodeList datosItem = item.getChildNodes();
 
                //Procesamos cada dato
                for (int j=0; j<datosItem.getLength(); j++)
                {
                    Node dato = datosItem.item(j);
                    String etiqueta = dato.getNodeName();
 
                    if (etiqueta.equals("id"))
                    {
                        String texto = obtenerTexto(dato);
                        aux.setId(Integer.parseInt(texto));
                    }
                    else if (etiqueta.equals("name"))
                    {
                         aux.setName(dato.getFirstChild().getNodeValue());
                    }
                    else if (etiqueta.equals("surname1"))
                    {
                         aux.setSurname(dato.getFirstChild().getNodeValue());
                    }
                    else if (etiqueta.equals("level"))
                    {
                         aux.setState(Integer.parseInt(dato.getFirstChild().getNodeValue()));
                    }
                    else if (etiqueta.equals("smsphone"))
                    {
                         aux.setSmsPhone(dato.getFirstChild().getNodeValue());
                    }
                    
                    else if (etiqueta.equals("photoarray"))
                    {
                        
                    	String datos = dato.getFirstChild().getNodeValue();
                    	if (datos.equals("null")){
                    		
                    	}
                    	else{
                    		byte[] b = hexStringToByteArray(datos);
                            aux.setPhoto(b);
                    	}
                    	
                    	
                    }
                }
 
                classes.add(aux);
            }
        }
        catch (Exception ex)
        {
            throw new RuntimeException(ex);
        }
 
        return  classes;
    }
 
    private String obtenerTexto(Node dato)
    {
        StringBuilder texto = new StringBuilder();
        NodeList fragmentos = dato.getChildNodes();
 
        for (int k=0;k<fragmentos.getLength();k++)
        {
            texto.append(fragmentos.item(k).getNodeValue());
        }
 
        return texto.toString();
    }
 
    private InputStream getInputStream()
    {
        try
        {
            return Url.openConnection().getInputStream();
        }
        catch (IOException e)
        {
            throw new RuntimeException(e);
        }
    }
    
    
    /**
     * Método que convierte un String hexadecimal a una array de bytes.
     * 
     * 
     * @param s
     * @return
     */
    public static byte[] hexStringToByteArray(String s) {
        int len = s.length();
        byte[] data = new byte[len / 2];
        for (int i = 0; i < len; i += 2) {
            data[i / 2] = (byte) ((Character.digit(s.charAt(i), 16) << 4)
                                 + Character.digit(s.charAt(i+1), 16));
        }
        return data;
    }

    
}